/*	proxylist.c -- 02/10/16 IF
**	output list (pair: inlet number & value)			  	
**	 
*/

#include "ext.h"			// Required for all Max external objects

void *this_class;			// Required. Global pointing to this class 
#define MAX 64

typedef struct _prox	// Data structure for this object
{
	t_object p_ob;		// Must always be the first field; used by Max
	void *p_proxy[MAX];
	int p_ints[MAX];
	short p_num;	// number of inlets
	long p_dummy;
	long p_inlet;	// inlet number
	void *p_out;		// Pointer to outlet
} t_prox;

// Prototypes for methods: you need a method for each message you want to respond to
void *prox_new(long num);
void prox_free(t_prox *x);
void prox_bang(t_prox *x);	// Method for message "int" in left inlet
void prox_int(t_prox *x, long n);
void prox_out(t_prox *x);

int main(void)
{
	// set up our class: create a class definition 
	setup((t_messlist **) &this_class, (method)prox_new, (method)prox_free, (short)sizeof(t_prox), 0L,
         A_DEFLONG, 0);	
	addbang((method)prox_bang);
	addint((method)prox_int);
	return (0);
}


void *prox_new(long num)
{
	t_prox *prox;
    int i;
    
	prox = (t_prox *)newobject(this_class);	// Create the new instance 
	if (num < 2)
		num = 2;
	prox->p_num = num; // total number of inlets (proxies)
    prox->p_out = listout(prox);	// Create outlet
    for (i = 0; i < MAX; i++)
    	prox->p_ints[i] = 0;
    for (i = num - 1; i > 0; i--)
      prox->p_proxy[i] = proxy_new(prox, (long)i, &prox-> p_dummy);
	return(prox);				// Must return a pointer to the new instance 
}

void prox_bang(t_prox *x)
{
	x->p_inlet = proxy_getinlet((t_object *)x);
	prox_out(x);
}


void prox_out(t_prox *x)
{
	t_atom myList[2];
//post("bang: %d %d", x->p_inlet, x->p_ints[x->p_inlet]);
	
	SETLONG(&myList[0], x->p_inlet);
	SETLONG(&myList[1], x->p_ints[x->p_inlet]);
	outlet_list(x->p_out, 0L, 2, myList);
}

void prox_int(t_prox *x, long n)
{
	x->p_inlet = proxy_getinlet((t_object *)x);
//post("int: %d %d", x->p_inlet, x->p_ints[x->p_inlet]);
	x->p_ints[x->p_inlet] = n;
	prox_out(x);
}

void prox_free(t_prox *x)
{
	short i;
	
	for (i = 1; i < x->p_inlet; i++)
		freeobject(x->p_proxy[i]);
}

